@@ -209,10 +209,21 @@ renv_lockfile_create_impl <- function(project, type, libpaths, packages, exclude
     project  = project
   )
 
-  # warn if some required packages are missing
-  ignored <- c(renv_project_ignored_packages(project), renv_packages_base(), exclude)
+  # check for missing packages
+  ignored <- c(renv_project_ignored_packages(project), renv_packages_base(), exclude, "renv")
   missing <- setdiff(packages, c(names(records), ignored))
-  if (!the$status_running && !the$init_running)
+
+  # cancel automatic snapshots if we have missing packages
+  if (length(missing) && the$auto_snapshot_running)
+    invokeRestart("cancel")
+
+  # give user a chance to handle missing packages, if any
+  #
+  # we only run this in top-level calls to snapshot() since renv will internally
+  # use snapshot() to create lockfiles, and missing packages are understood /
+  # tolerated there. this code mostly exists so interactive usages of snapshot()
+  # can recover and install missing packages
+  if (identical(topfun(), snapshot))
     renv_snapshot_report_missing(missing, type)
 
   records <- renv_snapshot_fixup(records)
