@@ -8,6 +8,11 @@ renv_methods_map <- function() {
       win32 = "renv_path_normalize_win32"
     ),
 
+    renv_file_exists = c(
+      unix  = "renv_file_exists_unix",
+      win32 = "renv_file_exists_win32"
+    ),
+
     renv_file_list_impl = c(
       unix  = "renv_file_list_impl_unix",
       win32 = "renv_file_list_impl_win32"
@@ -16,6 +21,11 @@ renv_methods_map <- function() {
     renv_file_broken = c(
       unix  = "renv_file_broken_unix",
       win32 = "renv_file_broken_win32"
+    ),
+
+    renv_paths_sandbox = c(
+      unix  = "renv_paths_sandbox_unix",
+      win32 = "renv_paths_sandbox_win32"
     )
 
   )
