@@ -1,13 +1,13 @@
 
-#' Re-Hash Packages in the renv Cache
+#' Re-hash packages in the renv cache
 #'
-#' Re-hash packages in the `renv` cache, ensuring that any previously-cached
+#' Re-hash packages in the renv cache, ensuring that any previously-cached
 #' packages are copied to a new cache location appropriate for this version of
-#' `renv`. This can be useful if the cache scheme has changed in a new version
-#' of `renv`, but you'd like to preserve your previously-cached packages.
+#' renv. This can be useful if the cache scheme has changed in a new version
+#' of renv, but you'd like to preserve your previously-cached packages.
 #'
 #' Any packages which are re-hashed will retain links to the location of the
-#' newly-hashed package, ensuring that prior installations of `renv` can still
+#' newly-hashed package, ensuring that prior installations of renv can still
 #' function as expected.
 #'
 #' @inheritParams renv-params
@@ -16,6 +16,7 @@
 rehash <- function(prompt = interactive(), ...) {
   renv_scope_error_handler()
   renv_dots_check(...)
+  renv_scope_verbose_if(prompt)
   invisible(renv_rehash_impl(prompt))
 }
 
@@ -37,13 +38,13 @@ renv_rehash_cache <- function(cache, prompt, action, label) {
   # re-compute package hashes
   old <- renv_cache_list(cache = cache)
 
-  vprintf("* Re-computing package hashes ... ")
+  printf("- Re-computing package hashes ... ")
   new <- map_chr(old, renv_progress_callback(renv_cache_path, length(old)))
-  vwritef("Done!")
+  writef("Done!")
 
   changed <- which(old != new & file.exists(old) & !file.exists(new))
   if (empty(changed)) {
-    vwritef("* Your cache is already up-to-date -- nothing to do.")
+    writef("- Your cache is already up-to-date -- nothing to do.")
     return(TRUE)
   }
 
@@ -53,17 +54,13 @@ renv_rehash_cache <- function(cache, prompt, action, label) {
     packages <- basename(old)[changed]
     oldhash <- renv_path_component(old[changed], 2L)
     newhash <- renv_path_component(new[changed], 2L)
-    renv_pretty_print(
-      sprintf(fmt, format(packages), format(oldhash), format(newhash)),
+    caution_bullets(
       "The following packages will be re-cached:",
-      sprintf("Packages will be %s to their new locations in the cache.", label),
-      wrap = FALSE
+      sprintf(fmt, format(packages), format(oldhash), format(newhash)),
+      sprintf("Packages will be %s to their new locations in the cache.", label)
     )
 
-    if (prompt && !proceed()) {
-      renv_report_user_cancel()
-      invokeRestart("abort")
-    }
+    cancel_if(prompt && !proceed())
 
   }
 
@@ -72,13 +69,13 @@ renv_rehash_cache <- function(cache, prompt, action, label) {
   names(sources) <- targets
   names(targets) <- sources
 
-  vprintf("* Re-caching packages ... ")
+  printf("- Re-caching packages ... ")
   enumerate(targets, renv_progress_callback(action, length(targets)))
-  vwritef("Done!")
+  writef("Done!")
 
   n <- length(targets)
   fmt <- "Successfully re-cached %s."
-  vwritef(fmt, nplural("package", n))
+  writef(fmt, nplural("package", n))
 
   renv_cache_clean_empty()
 
